@@ -70,7 +70,7 @@ TEST_ALL_TEMPLATES = None
 SYS_ADMIN_LOGIN_CMD = None
 ORG_ADMIN_LOGIN_CMD = None
 VAPP_AUTHOR_LOGIN_CMD = None
-USER_LOGOUT_CMD = f"logout"
+USER_LOGOUT_CMD = "logout"
 USERNAME_TO_LOGIN_CMD = {}
 USERNAME_TO_CLUSTER_NAME = {}
 
@@ -203,11 +203,11 @@ def create_user(username, password, role):
     # no assert here because if the user exists, the exit code will be 2
 
     # user can already exist but be disabled
-    cmd = f"user update {username} --enable"
-    result = CLI_RUNNER.invoke(vcd, cmd.split(), catch_exceptions=False)
-    assert result.exit_code == 0,\
-        testutils.format_command_info('vcd', cmd, result.exit_code,
-                                      result.output)
+    # cmd = f"user update {username} --enable"
+    # result = CLI_RUNNER.invoke(vcd, cmd.split(), catch_exceptions=False)
+    # assert result.exit_code == 0,\
+    #    testutils.format_command_info('vcd', cmd, result.exit_code,
+    #                                  result.output)
 
 
 def delete_catalog_item(item_name):
